# encoding: utf-8
"""
@project: djangoModel->enroll_statistics
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 报名统计服务
@created_time: 2023/2/22 14:31
"""
import datetime

from django.core.paginator import Paginator, EmptyPage
from django.db.models import Count, Sum, Avg, Q
from django.db.models.functions import TruncDay, Round

from xj_user.services.user_relate_service import UserRelateToUserService
from xj_user.services.user_service import UserService
from ..models import Enroll, EnrollRecord, EnrollSubitemRecord
from ..utils.custom_tool import format_params_handle
from ..utils.join_list import JoinList


class EnrollStatisticsServices(object):
    """报名主项统计服务(发布人统计)"""

    @staticmethod
    def every_one_total(params=None, need_Pagination=True):
        """
        每个人汇总统计报名表（后台展示），可以根据事件和状态码筛选。
        :param params:
        :return: [{
        "user_id":用户ID,
        "user_name": 用户名称,
        "nick_name": 用户名称,
        "count":"发布次数",
        "amount_total":"总收合计"，
        "paid_amount_total":"为首合计",
        "paid_amount_total":"以收合计",
        "average_price":"平均价格",
        "commision_total":"总佣金",
        "counts_total":"份数总计"
        }]，err_msg
        """
        if params is None:
            params = {}
        page = params.get("page", 1)
        size = params.get("size", 10)
        # 字段过滤
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                # ID 编码 code 等等筛选
                "enroll_id", "thread_id", "category_id", "user_id", "enroll_rule_group_id", "trading_relate", "region_code",
                "occupy_room", "enroll_status_code", "user_relate_type_id",
                # 布尔条件筛选
                "need_vouch", "need_deposit", "need_imprest", "enable_pool", "hide_price", "hide_user", "has_repeat",
                "has_subitem", "has_audit",
                # 时间区间筛选 注意：所有的数值区间参数开始start后缀开始，以end结尾。
                "open_time", "close_time", "launch_time", "finish_time", "spend_time", "create_time", "update_time",
                "open_time_start", "close_time_start", "launch_time_start", "finish_time_start", "spend_time_start", "create_time_start", "update_time_start",
                "open_time_end", "close_time_end", "launch_time_end", "finish_time_end", "spend_time_end", "create_time_end", "update_time_end",
                # 数值区间筛选，注意：所有的数值区间参数开始min后缀开始，以max结尾。
                "min_number", "max_number", "max_count_apiece", "min_count_apiece",  # 不做区间处理
                "price", "count", "fee", "subitems_amount", "amount", "paid_amount", "unpaid_amount", "commision", "reduction", "pool_limit", "pool_stopwatch", "deposit",
                "price_min", "count_min", "fee_min", "subitems_amount_min", "amount_min", "paid_amount_min", "unpaid_amount_min", "commision_min", "reduction_min", "pool_limit_min",
                "pool_stopwatch_min", "deposit_min",
                "price_max", "count_max", "fee_max", "subitems_amount_max", "amount_max", "paid_amount_max", "unpaid_amount_max", "commision_max", "reduction_max", "pool_limit_max",
                "pool_stopwatch_max", "deposit_max",
            ],
            remove_filed_list=None,
            alias_dict={
                "enroll_id": "id",
                # 时间区间还原成orm表达式
                "open_time_start": "open_time__gte", "close_time_start": "close_time__gte", "launch_time_start": "launch_time__gte", "finish_time_start": "finish_time__gte",
                "spend_time_start": "spend_time__gte", "create_time_start": "create_time__gte", "update_time_start": "update_time__gte", "open_time_end": "open_time__lte",
                "close_time_end": "close_time__lte", "launch_time_end": "launch_time__lte", "finish_time_end": "finish_time__lte", "spend_time_end": "spend_time__lte",
                "create_time_end": "create_time__lte", "update_time_end": "update_time__lte",
                # 数值区间还原成orm表达式
                "price_min": "price__gte", "count_min": "count__gte", "fee_min": "fee__gte", "subitems_amount_min": "subitems_amount__gte", "amount_min": "amount__gte",
                "paid_amount_min": "paid_amount__gte", "unpaid_amount_min": "unpaid_amount__gte", "commision_min": "commision__gte", "reduction_min": "reduction__gte",
                "pool_limit_min": "pool_limit__gte", "pool_stopwatch_min": "pool_stopwatch__gte", "deposit_min": "deposit__gte",
                "price_max": "price__lte", "count_max": "count__lte", "fee_max": "fee__lte", "subitems_amount_max": "subitems_amount__lte", "amount_max": "amount__lte",
                "paid_amount_max": "paid_amount__lte", "unpaid_amount_max": "unpaid_amount__lte", "commision_max": "commision__lte", "reduction_max": "reduction__lte",
                "pool_limit_max": "pool_limit__lte", "pool_stopwatch_max": "pool_stopwatch__lte", "deposit_max": "deposit__lte",
            },
            is_remove_null=True
        )

        # 基础条件筛选出大致范围
        search_obj = Enroll.objects.filter(**params)

        # 用户关系搜索， TODO 如果后期 with_user_id_list大于2500个mysql会报错。所以这里存在性能优化空间
        user_relate_type_id = params.get("user_relate_type_id", None)
        user_id = params.get("user_id", None)
        if user_relate_type_id and user_id:
            relate_user_list, err = UserRelateToUserService.list(params={"need_pagination": 0, "user_id": user_id, "user_relate_type_id": user_relate_type_id})
            with_user_id_list = [i["with_user_id"] for i in relate_user_list]
            search_obj.filter(user_id__in=with_user_id_list)

        # 进行group分组计算
        group_obj = search_obj.values("user_id").annotate(
            push_count=Count("id"),
            average_price=Round(Avg("price")),
            amount_total=Round(Sum("amount")),
            paid_amount_total=Round(Sum("paid_amount")),
            unpaid_amount_total=Round(Sum("unpaid_amount")),
            commision_total=Round(Sum("commision")),
            count_total=Round(Sum("count"))
        )
        group_obj = group_obj.values(
            "user_id", "push_count", "average_price", "amount_total", "paid_amount_total", "unpaid_amount_total", "commision_total", "count_total"
        )
        total = group_obj.count()

        if need_Pagination:
            # 进行分页查询
            paginator = Paginator(group_obj, size)
            try:
                enroll_obj = paginator.page(page)
            except EmptyPage:
                enroll_obj = paginator.page(paginator.num_pages)
            except Exception as e:
                return None, f'{str(e)}'
            group_obj = enroll_obj

        # 报名汇总
        enroll_total_list = list(group_obj)
        # 查询 用户信息
        user_ids = [i["user_id"] for i in enroll_total_list if i["user_id"]]
        user_info_list, err = UserService.user_search(user_id_list=user_ids, need_map=False, filter_fields=["id", "user_name", "nickname", "phone"])
        # 拼接 用户信息
        enroll_total_list = JoinList(enroll_total_list, user_info_list, "user_id", "user_id").join()

        return {"page": page, "size": size, "total": total, "list": enroll_total_list, } if need_Pagination else enroll_total_list, None

    @staticmethod
    def every_day_total(params=None, need_Pagination=True):
        """
        每个人汇总统计，可以根据事件和状态码筛选。
        :param params:
        :return: data，err_msg
        """

        if params is None:
            params = {}
        page = params.get("page", 1)
        size = params.get("size", 10)
        today = datetime.date.today()
        last_thirty_day = today - datetime.timedelta(days=30)

        # 默认仅仅差30天内的数据
        params.setdefault("create_time_start", last_thirty_day.strftime("%Y-%m-%d 00:00:00"))
        params.setdefault("create_time_end", today.strftime("%Y-%m-%d 23:59:59"))
        create_time_start = params.get("create_time_start", None)
        create_time_end = params.get("create_time_end", None)

        # 字段过滤
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                # ID 编码 code 等等筛选
                "enroll_id", "thread_id", "category_id", "user_id", "enroll_rule_group_id", "trading_relate", "region_code",
                "occupy_room", "enroll_status_code", "user_relate_type_id",
                # 布尔条件筛选
                "need_vouch", "need_deposit", "need_imprest", "enable_pool", "hide_price", "hide_user", "has_repeat",
                "has_subitem", "has_audit",
                # 时间区间筛选 注意：所有的数值区间参数开始start后缀开始，以end结尾。
                "open_time", "close_time", "launch_time", "finish_time", "spend_time", "create_time", "update_time",
                "open_time_start", "close_time_start", "launch_time_start", "finish_time_start", "spend_time_start", "create_time_start", "update_time_start",
                "open_time_end", "close_time_end", "launch_time_end", "finish_time_end", "spend_time_end", "create_time_end", "update_time_end",
                # 数值区间筛选，注意：所有的数值区间参数开始min后缀开始，以max结尾。
                "min_number", "max_number", "max_count_apiece", "min_count_apiece",  # 不做区间处理
                "price", "count", "fee", "subitems_amount", "amount", "paid_amount", "unpaid_amount", "commision", "reduction", "pool_limit", "pool_stopwatch", "deposit",
                "price_min", "count_min", "fee_min", "subitems_amount_min", "amount_min", "paid_amount_min", "unpaid_amount_min", "commision_min", "reduction_min", "pool_limit_min",
                "pool_stopwatch_min", "deposit_min",
                "price_max", "count_max", "fee_max", "subitems_amount_max", "amount_max", "paid_amount_max", "unpaid_amount_max", "commision_max", "reduction_max", "pool_limit_max",
                "pool_stopwatch_max", "deposit_max",
            ],
            remove_filed_list=None,
            alias_dict={
                "enroll_id": "id",
                # 时间区间还原成orm表达式
                "open_time_start": "open_time__gte", "close_time_start": "close_time__gte", "launch_time_start": "launch_time__gte", "finish_time_start": "finish_time__gte",
                "spend_time_start": "spend_time__gte", "create_time_start": "create_time__gte", "update_time_start": "update_time__gte", "open_time_end": "open_time__lte",
                "close_time_end": "close_time__lte", "launch_time_end": "launch_time__lte", "finish_time_end": "finish_time__lte", "spend_time_end": "spend_time__lte",
                "create_time_end": "create_time__lte", "update_time_end": "update_time__lte",
                # 数值区间还原成orm表达式
                "price_min": "price__gte", "count_min": "count__gte", "fee_min": "fee__gte", "subitems_amount_min": "subitems_amount__gte", "amount_min": "amount__gte",
                "paid_amount_min": "paid_amount__gte", "unpaid_amount_min": "unpaid_amount__gte", "commision_min": "commision__gte", "reduction_min": "reduction__gte",
                "pool_limit_min": "pool_limit__gte", "pool_stopwatch_min": "pool_stopwatch__gte", "deposit_min": "deposit__gte",
                "price_max": "price__lte", "count_max": "count__lte", "fee_max": "fee__lte", "subitems_amount_max": "subitems_amount__lte", "amount_max": "amount__lte",
                "paid_amount_max": "paid_amount__lte", "unpaid_amount_max": "unpaid_amount__lte", "commision_max": "commision__lte", "reduction_max": "reduction__lte",
                "pool_limit_max": "pool_limit__lte", "pool_stopwatch_max": "pool_stopwatch__lte", "deposit_max": "deposit__lte",
            },
            is_remove_null=True
        )

        # 基础条件筛选出大致范围
        search_obj = Enroll.objects.filter(**params)

        # 用户关系搜索， TODO 如果后期 with_user_id_list大于2500个mysql会报错。所以这里存在性能优化空间
        user_relate_type_id = params.get("user_relate_type_id", None)
        user_id = params.get("user_id", None)
        if user_relate_type_id and user_id:
            relate_user_list, err = UserRelateToUserService.list(params={"need_pagination": 0, "user_id": user_id, "user_relate_type_id": user_relate_type_id})
            with_user_id_list = [i["with_user_id"] for i in relate_user_list]
            search_obj = search_obj.filter(user_id__in=with_user_id_list)

        # 进行group分组计算
        group_obj = search_obj.annotate(create_date=TruncDay("create_time")).values("create_date").annotate(
            push_count=Count("id"),
            average_price=Round(Avg("price")),
            amount_total=Round(Sum("amount")),
            paid_amount_total=Round(Sum("paid_amount")),
            unpaid_amount_total=Round(Sum("unpaid_amount")),
            commision_total=Round(Sum("commision")),
            count_total=Round(Sum("count"))
        ).order_by("-create_date")
        group_obj = group_obj.values(
            "create_date", "push_count", "amount_total", "average_price", "paid_amount_total", "unpaid_amount_total", "commision_total", "count_total"
        )
        total = group_obj.count()

        if need_Pagination:
            # 进行分页查询
            paginator = Paginator(group_obj, size)
            try:
                enroll_obj = paginator.page(page)
            except EmptyPage:
                enroll_obj = paginator.page(paginator.num_pages)
            except Exception as e:
                return None, f'{str(e)}'
            group_obj = enroll_obj

        # 报名汇总
        enroll_total_list = list(group_obj)
        return {
                   "page": page,
                   "size": size,
                   "total": total,
                   "create_time_start": create_time_start,
                   "create_time_end": create_time_end,
                   "list": enroll_total_list,
               } if need_Pagination else enroll_total_list, None

    @staticmethod
    def statistics_by_day():
        """
        小程序临时接口
        :return:
        """
        this_day = str(datetime.date.today())
        this_day_start = this_day + " 00:00:00"
        this_day_end = this_day + " 23:59:59"
        before_seven_day_start = ((datetime.datetime.now()) + datetime.timedelta(days=-7)).strftime("%Y-%m-%d") + " 00:00:00"
        result = {}
        # 今日成交数量
        # this_day_obj_count = Enroll.objects.filter(Q(enroll_status_code=680) | Q(enroll_status_code=80)).filter(create_time__gt=this_day_start, create_time__lt=this_day_end).count()
        this_day_obj_count = Enroll.objects.filter(create_time__gt=this_day_start, create_time__lt=this_day_end).count()
        result["this_day_count"] = this_day_obj_count
        # 近七日的成交数量与金额
        # nearly_seven_days = list(
        #     Enroll.objects.filter(Q(enroll_status_code=680) | Q(enroll_status_code=80))
        #         .filter(create_time__gt=before_seven_day_start, create_time__lt=this_day_end) \
        #         .annotate(created_day=TruncDay("create_time")) \
        #         .values("created_day") \
        #         .annotate(deel_count=Count("id"))
        #         .annotate(deel_amount=Sum("amount"))
        #         .values("created_day", "deel_count", "deel_amount")
        # )

        nearly_seven_days = list(
            Enroll.objects.filter(create_time__gt=before_seven_day_start, create_time__lt=this_day_end) \
                .annotate(created_day=TruncDay("create_time")) \
                .values("created_day") \
                .annotate(deel_count=Count("id"))
                .annotate(deel_amount=Sum("amount"))
                .values("created_day", "deel_count", "deel_amount")
        )
        day_map = {}
        for i in nearly_seven_days:
            i["created_day"] = i["created_day"].strftime('%Y-%m-%d')
            i["deel_amount"] = round(float(i["deel_amount"]), 2) if i["deel_amount"] else 0
            day_map[i["created_day"]] = i
        nearly_seven_days_list = []
        for i in range(7):
            this_day = ((datetime.datetime.now()) + datetime.timedelta(days=-i)).strftime("%Y-%m-%d")
            nearly_seven_days_list.append(day_map.get(this_day) if day_map.get(this_day) else {"created_day": this_day, "deel_amount": 0, "deel_count": 0})
        result["nearly_seven_days"] = nearly_seven_days_list

        # 总成交金额和总的成交数量
        # total_deel_amount = Enroll.objects.filter(Q(enroll_status_code=680) | Q(enroll_status_code=80)).aggregate(total_deel_amount=Sum('amount'))
        # total_deel_count = Enroll.objects.filter(Q(enroll_status_code=680) | Q(enroll_status_code=80)).aggregate(total_deel_count=Count('id'))

        total_deel_amount = Enroll.objects.filter().aggregate(total_deel_amount=Sum('amount'))
        total_deel_count = Enroll.objects.filter().aggregate(total_deel_count=Count('id'))

        result["total_deel_amount"] = total_deel_amount["total_deel_amount"] if total_deel_amount["total_deel_amount"] else 0
        result["total_deel_count"] = total_deel_count["total_deel_count"] if total_deel_count["total_deel_count"] else 0

        # 代发佣金和提成
        await_commission_main = EnrollRecord.objects.exclude(Q(enroll_status_code=680) | Q(enroll_status_code=80)).aggregate(total_deel_amount=Sum('amount'))
        await_commission_subitem = EnrollSubitemRecord.objects.exclude(Q(enroll_subitem_status_code=680) | Q(enroll_subitem_status_code=80)).aggregate(total_subitem_amount=Sum('subitem_amount'))

        already_commission_main = EnrollRecord.objects.filter(Q(enroll_status_code=680) | Q(enroll_status_code=80)).aggregate(total_deel_amount=Sum('amount'))
        already_commission_subitem = EnrollSubitemRecord.objects.filter(Q(enroll_subitem_status_code=680) | Q(enroll_subitem_status_code=80)).aggregate(total_subitem_amount=Sum('subitem_amount'))
        result["await_commission_main"] = round(await_commission_main["total_deel_amount"], 2) if await_commission_main["total_deel_amount"] else 0
        result["await_commission_subitem"] = round(await_commission_subitem["total_subitem_amount"], 2) if await_commission_subitem["total_subitem_amount"] else 0
        result["already_commission_main"] = round(already_commission_main["total_deel_amount"], 2) if already_commission_main["total_deel_amount"] else 0
        result["already_commission_subitem"] = round(already_commission_subitem["total_subitem_amount"], 2) if already_commission_subitem["total_subitem_amount"] else 0

        return result, None

    @staticmethod
    def statistics_by_user(user_id=None):
        """
        小程序临时统计接口
        :param user_id:
        :return:
        """
        if not user_id:
            return None, "user_id 不能为空"
        this_day = str(datetime.date.today())
        this_day_start = this_day + " 00:00:00"
        this_day_end = this_day + " 23:59:59"
        today_order_num = Enroll.objects.filter(user_id=user_id).filter(create_time__gt=this_day_start, create_time__lt=this_day_end).count()
        # .filter(Q(enroll_status_code=680) | Q(enroll_status_code=80)) \

        today__undertake_num = EnrollRecord.objects.filter(user_id=user_id) \
            .filter(create_time__gt=this_day_start, create_time__lt=this_day_end, ) \
            .exclude(enroll_status_code=124).count()
        # .filter(Q(enroll_status_code=680) | Q(enroll_status_code=80))
        total_order_num = Enroll.objects.filter(user_id=user_id).count()
        total__undertake_num = EnrollRecord.objects.filter(user_id=user_id).exclude(enroll_status_code=124).count()

        return {"today_order_num": today_order_num, "today__undertake_num": today__undertake_num, "total_order_num": total_order_num, "total__undertake_num": total__undertake_num}, None



# encoding: utf-8
"""
@project: djangoModel->enroll_record_serivce
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户报名记录
@created_time: 2022/9/17 15:45
"""
from django.core.paginator import Paginator, EmptyPage
from django.db.models import F

from xj_thread.services.thread_item_service import ThreadItemService
from xj_thread.services.thread_list_service import ThreadListService
from xj_user.services.user_detail_info_service import DetailInfoService
from ..models import EnrollRecord, Enroll, EnrollSubitemRecord, EnrollRecordExtendField, EnrollSubitem
from ..serializers import EnrollRecordListSerializer, EnrollRecordListV2Serializer
from ..service.enroll_status_code_service import EnrollStatusCodeService
from ..service.enroll_subitem_record_service import EnrollSubitemRecordService
from ..service.valuation_service import ValuationService
from ..utils.custom_tool import format_params_handle, filter_result_field, write_to_log, force_transform_type, filter_fields_handler, conflict_fields
from ..utils.join_list import JoinList


class EnrollRecordServices:
    enroll_record_fields = [i.name for i in EnrollRecord._meta.fields]
    enroll_record_extend_fields = [i.name for i in EnrollRecord._meta.fields if "field_" in i.name]
    enroll_record_except_extend_fields = [i.name for i in EnrollRecord._meta.fields if not "field_" in i.name]
    enroll_record_validate_type_fields = [
        "id|int", "enroll_id|int", "apply_subitem_ids|list", "user_id|int", "enroll_auth_status", "enroll_pay_status",
        "enroll_status_code|int", "create_time|date", "price|float", "deposit|float", "count|int", "main_amount|float",
        "coupon_amount|float", "again_reduction|float", "subitems_amount|float", "deposit_amount|float", "amount|float",
        "paid_amount|float", "unpaid_amount|float", "fee|float", "photos|dict", "files|dict", "score|float", "reply",
        "remark", "finish_time|date", "again_price|float", "again_fee|float", "initiator_again_price|float",
        "estimated_time|date", "update_time|date",
        "thread_id_list|list_int",
        "enroll_id_list|list_int",
        "enroll_status_code_list|list_int",
    ]

    @staticmethod
    def extend_transform_fields(result_list: list = None):
        """
        报名记录列表的扩展字段替换过滤
        :param result_list:
        :return: result_list
        """
        filed_map_list = list(EnrollRecordExtendField.objects.all().values("field", 'field_index'))
        filed_map = {item['field_index']: item['field'] for item in filed_map_list}

        # 扩展字段替换，并过滤
        enroll_record_list = filter_result_field(
            result_list=result_list,
            alias_dict=filed_map
        )
        enroll_record_list = filter_result_field(
            result_list=enroll_record_list,
            remove_filed_list=["field_1", "field_2", "field_3", "field_4", "field_5", "field_6", "field_7", "field_8", "field_9", "field_10"]
        )
        return enroll_record_list

    @staticmethod
    def check_can_add(enroll_id: int = None, user_id: int = None, apply_subitem_ids: list = None, **kwargs):
        """
        判断是否可以报名
        :param apply_subitem_ids: 期待申请分项ID列表
        :param enroll_id: 报名ID
        :param user_id: 用户ID
        :return: Bool
        """

        enroll_id, is_pass = force_transform_type(variable=enroll_id, var_type="int")
        user_id, is_pass = force_transform_type(variable=user_id, var_type="int")
        if not enroll_id or not user_id:
            return None, "参数错误"

        # 状态码常量
        draft_code, is_pass = force_transform_type(variable=kwargs.get("draft_code", 124), var_type="int", default=124)  # 草稿code
        enrolled_code, is_pass = force_transform_type(variable=kwargs.get("enrolled_code", 356), var_type="int", default=356)  # 已报名code
        enrolling_code, is_pass = force_transform_type(variable=kwargs.get("enrolling_code", 234), var_type="int", default=234)  # 报名中code

        # 判断报名ID是否正确
        enroll_obj = Enroll.objects.filter(id=enroll_id)
        if enrolling_code:
            enroll_obj = enroll_obj.filter(enroll_status_code=enrolling_code)
        enroll_obj = enroll_obj.first()
        if not enroll_obj:
            return False, "当前报名项不可报名"

        # 当前用户报名过了，不允许报名了
        enroll_record_obj = EnrollRecord.objects.filter(enroll_id=enroll_id, user_id=user_id)  # 拿到搜索报名记录
        if draft_code:
            enroll_record_obj = enroll_record_obj.exclude(enroll_status_code=draft_code)
        this_user_record_obj = enroll_record_obj.first()
        if this_user_record_obj:
            return False, "已经报名，无需再次报名"

        # 报名分项ID列表
        apply_subitem_ids, is_pass = force_transform_type(variable=apply_subitem_ids, var_type="list_int")
        if not apply_subitem_ids:
            return True, None
        enroll_subitem_ids = list(EnrollSubitem.objects.filter(enroll_id=enroll_id).values("id"))
        enroll_subitem_ids = [i.get("id") for i in enroll_subitem_ids]
        for apply_subitem_id in apply_subitem_ids:
            if not apply_subitem_id in enroll_subitem_ids:
                return False, "您期望报名的分项中存在无效的分项,请重新选择您期望报名的分项"
        return True, None

    @staticmethod
    def check_can_cancel(pk, change_code=None, should_code=None):
        """
        检查报名记录是否可以修改为草稿
        :param pk: 主键ID
        :return: data, err
        """
        if change_code and should_code and str(change_code) == "124":
            current_status_code = EnrollRecord.objects.filter(id=pk).values("enroll_status_code").first()
            current_status_code = current_status_code.get("enroll_status_code", 0)
            if str(should_code) == str(current_status_code):
                return True, None
            else:
                return False, None

        return True, None

    @staticmethod
    def record_add(params: dict = None):
        """
        报名添加
        :param params: request解析出来的json参数
        :return: 报名记录json
        """
        # ==============section 判断是否可以继续报名 start ====================
        params, is_pass = force_transform_type(variable=params, var_type="dict")
        enroll_id, is_pass = force_transform_type(variable=params.get("enroll_id"), var_type="int")
        user_id, is_pass = force_transform_type(variable=params.get("user_id"), var_type="int")
        apply_subitem_ids, is_pass = force_transform_type(variable=params.get("apply_subitem_ids", None), var_type="list_int")
        if not enroll_id:
            return None, "enroll_id不能为空"
        # ==============section 判断是否可以继续报名 end   ====================

        # ==============section 查看是否符合报名条件 start ====================
        res, err_msg = EnrollRecordServices.check_can_add(
            enroll_id=enroll_id,
            user_id=user_id,
            apply_subitem_ids=apply_subitem_ids,
            draft_code=params.pop("draft_code", 124),
            enrolling_code=params.pop("enrolling_code", 234),
            enrolled_code=params.pop("enrolled_code", 356)
        )
        if not res:
            return None, err_msg
        # ==============section 查看是否符合报名条件 end   ====================

        # ==============section 扩展字段替换，字段合法性验证 start ====================
        filed_map_list = list(EnrollRecordExtendField.objects.all().values("field", 'field_index'))
        reversal_filed_map = {
            item['field']: item['field_index'] for item in filed_map_list if item['field_index'] in EnrollRecordServices.enroll_record_extend_fields
        }
        try:
            params = format_params_handle(
                param_dict=params,
                is_validate_type=True,
                alias_dict=reversal_filed_map,
                is_remove_empty=True,
                filter_filed_list=list(reversal_filed_map.keys()) + [
                    "enroll_id|int",
                    "apply_subitem_ids|list_int",
                    "user_id|int",
                    "enroll_auth_status|int",
                    "enroll_pay_status|int",
                    "enroll_status_code|int",
                    "create_time|date",
                    "price|float",
                    "deposit|float",
                    "count|int",
                    "main_amount|float",
                    "coupon_amount|float",
                    "again_reduction|float",
                    "subitems_amount|float",
                    "deposit_amount|float",
                    "amount|float",
                    "paid_amount|float",
                    "unpaid_amount|float",
                    "fee|float",
                    "photos|json",
                    "files|json",
                    "score|float",
                    "reply",
                    "remark",
                    "finish_time|date",
                    "again_price|float",
                    "again_fee|float",
                    "initiator_again_price|float",
                    "estimated_time|date",
                    "update_time|date",
                ],
            )
        except ValueError as e:
            return None, str(e)

        # 必填字段验证
        must_keys = ["enroll_id", "user_id"]
        for i in must_keys:
            if not params.get(i):
                return None, str(i) + "为必填"
        # ==============section 扩展字段替换，字段合法性验证 end  ====================
        # IO 操作
        try:
            instance = EnrollRecord.objects.create(**params)
            add_info = instance.to_json()
        except Exception as e:
            return None, str(e)
        return {"enroll_id": add_info.get("enroll_id"), "id": add_info.get("id")}, None

    @staticmethod
    def record_list(params: dict, need_pagination: bool = True, exclude_code=124, **kwargs):
        """
        查询报名记录列表
        :param exclude_code: 不查看某状态码（enroll_status_code）
        :param params: 查询参数
        :param need_pagination: 是否分页
        :return: 报名记录
        """
        page, is_pass = force_transform_type(variable=params.pop('page', 1), var_type="int", default=1)
        size, is_pass = force_transform_type(variable=params.pop('size', 10), var_type="int", default=10)
        sort = params.pop('sort', "-id")
        sort = sort if sort in [
            "id", "-id", "-update_time", "update_time", "finish_time", "-finish_time", "create_time", "-create_time",
            "estimated_time", "-estimated_time"
        ] else "-id"
        # ============= section 根据信息搜索信息ID列表来检索报名记录 ===========================
        thread_search_prams = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "title", "subtitle", "access_level", "author",
                "has_enroll", "has_fee", "has_comment", "has_location", "is_original", "finance_invoicing_code",
                "category_value", "classify_value", "thread_category_value", "thread_classify_value",
                "platform_code", "need_auth", "show_value"
            ],
        )
        if thread_search_prams:
            params["thread_id_list"], err = ThreadListService.search_ids(thread_search_prams, is_strict_mode=False)
        # ============= section 根据信息搜索信息ID列表来检索报名记录 ===========================

        # ============= section 扩展字段映射获取，扩展字段查询前 筛选替换 start ===========================
        filed_map_list = list(EnrollRecordExtendField.objects.all().values("field", 'field_index'))
        filed_map = {item['field_index']: item['field'] for item in filed_map_list}
        reversal_filed_map = {item['field']: item['field_index'] for item in filed_map_list}

        # 添加构建ORM的查询参数
        reversal_filed_map.update({
            "category_id": "enroll__category_id",
            "enroll_user_id": "enroll__user_id",
            "thread_id": "enroll__thread_id",
            "enroll": "enroll_id",
            "enroll_id": "enroll_id",
            "thread_id_list": "enroll__thread_id__in",
            "enroll_id_list": "enroll_id__in",
            "enroll_status_code_list": "enroll_status_code__in"
        })

        params = format_params_handle(
            param_dict=params,
            is_remove_empty=True,
            filter_filed_list=list(reversal_filed_map.keys()) + EnrollRecordServices.enroll_record_validate_type_fields,
            split_list=["enroll_status_code_list", "enroll_id_list", "thread_id_list"],
            alias_dict=reversal_filed_map
        )
        # ============= section 扩展字段映射获取，扩展字段查询前 筛选替换 end    ===========================

        # ==================== section 构建ORM条件查询 start===============================
        record_obj = EnrollRecord.objects.order_by(sort).annotate(
            thread_id=F("enroll__thread_id"),
            enroll_remark=F("enroll__remark")
        ).filter(**params)
        exclude_code, is_pass = force_transform_type(variable=exclude_code, var_type="int")
        if exclude_code:
            record_obj = record_obj.exclude(enroll_status_code=124)
        total = record_obj.count()
        if not need_pagination:
            enroll_list = EnrollRecordListSerializer(record_obj, many=True).data
        else:
            paginator = Paginator(record_obj, size)
            try:
                record_obj = paginator.page(page)
            except EmptyPage:
                return {'total': total, "page": page, "size": size, 'list': []}, None
            enroll_list = EnrollRecordListSerializer(record_obj, many=True).data

        # ==================== section 构建ORM条件查询 end   ===============================

        # 扩展字段替换，并过滤没有配置扩展字段
        enroll_list = filter_result_field(
            result_list=filter_result_field(
                result_list=enroll_list,
                alias_dict=filed_map,  # 替换已经配置的扩展字段
            ),
            remove_filed_list=EnrollRecordServices.enroll_record_extend_fields  # 去掉没有配置的扩展字段
        )

        # ==================== 拼接跨服务获取信息 start===============================
        try:
            # 拼接用户信息
            user_info_list = DetailInfoService.get_list_detail(
                None,
                user_id_list=[i["user_id"] for i in enroll_list if i.get("user_id")],
                filter_fields=["user_id", "real_name", "user_name", "full_name", "nickname", "phone"]
            )
            user_info_list = conflict_fields(source_data=enroll_list, foreign_data=user_info_list, prefix="user_")
            JoinList(enroll_list, user_info_list, l_key="user_id", r_key="user_user_id").join()

            # 拼接Thread信息
            thread_list, err = ThreadListService.search(
                id_list=[i["thread_id"] for i in enroll_list if i.get("thread_id")],
                filter_fields=[
                    "id", "title", "subtitle", "summary", "author", "category_value",
                    "category_name", "classify_value", "classify_name", "remark"
                ]
            )
            thread_list = conflict_fields(source_data=enroll_list, foreign_data=thread_list, prefix="thread_")
            JoinList(enroll_list, thread_list, l_key="thread_id", r_key="thread_id").join()
        except Exception as e:
            write_to_log(prefix="报名记录列表服务拼接数据异常", err_obj=e)
        # ==================== 拼接跨服务获取信息 end   ===============================

        # 返回数据
        return enroll_list if not need_pagination else {'total': total, "page": page, "size": size, 'list': enroll_list}, None

    # note 服务即将被 上面的record_list代替
    @staticmethod
    def complex_record_list(params: dict = None):
        """
        这个接口仅仅提供API使用,不提供服务调用，关联太多的数据，使用了很多的where_in,所以限制分页
        :param params:
        :param need_pagination:
        :return:
        """
        # 参数预处理
        params, is_pass = force_transform_type(variable=params, var_type="int", default={})
        page, is_pass = force_transform_type(variable=params.pop('page', 1), var_type="int", default=1)
        size, is_pass = force_transform_type(variable=params.pop('size', 10), var_type="int", default=10)
        if size > 100:
            return None, "分布不可以超过100页"
        # 根据信息搜索信息ID列表来检索报名记录
        thread_search_prams = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "title", "subtitle", "access_level", "author",
                "has_enroll", "has_fee", "has_comment", "has_location", "is_original", "finance_invoicing_code",
                "category_value", "classify_value", "thread_category_value", "thread_classify_value",
                "platform_code", "need_auth", "show_value"
            ],
        )
        if thread_search_prams:
            params["thread_id_list"], err = ThreadListService.search_ids(thread_search_prams, is_strict_mode=False)

        # 扩展字段映射获取
        filed_map_list = list(EnrollRecordExtendField.objects.all().values("field", 'field_index'))
        forward_filed_map = {item['field_index']: item['field'] for item in filed_map_list}
        reversal_filed_map = {item['field']: item['field_index'] for item in filed_map_list}
        filter_filed_list = list(forward_filed_map.values()) + [
            "thread_id_list", "id", "category_id", "enroll_id", "user_id", "enroll_auth_status_id", "enroll_pay_status_id", "enroll_status_code", "create_time",
            "price", "deposit", "count", "main_amount", "coupon_amount", "again_reduction", "subitems_amount", "deposit_amount", "amount",
            "paid_amount", "unpaid_amount", "fee", "photos", "files", "score", "reply", "remark", "finish_time",
            "again_price", "again_fee", "initiator_again_price", "estimated_time", "update_time",
        ]
        reversal_filed_map.update({
            "thread_id_list": "enroll__thread_id__in",
            "category_id": "enroll__category_id",
            "enroll": "enroll_id"
        })

        # 扩展字段搜索项筛选
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=filter_filed_list,
            alias_dict=reversal_filed_map
        )
        # PS 124状态码任务取消报名，不予显示
        enroll_obj = EnrollRecord.objects.filter(**params).annotate(
            thread_id=F("enroll__thread_id"),
            enroll_user_id=F("enroll__user_id"),
        ).exclude(enroll_status_code=124)
        count = enroll_obj.count()

        # 是否分页
        paginator = Paginator(enroll_obj, size)
        try:
            enroll_obj = paginator.page(page)
        except EmptyPage:
            enroll_obj = paginator.page(paginator.num_pages)
        enroll_list = EnrollRecordListV2Serializer(enroll_obj, many=True).data

        # 扩展字段替换，并过滤
        enroll_list = filter_result_field(
            result_list=filter_result_field(
                result_list=enroll_list,
                alias_dict=forward_filed_map
            ),
            remove_filed_list=["field_1", "field_2", "field_3", "field_4", "field_5", "field_6", "field_7", "field_8", "field_9", "field_10"]
        )
        # 拼接报名用户信息
        user_info_map = {i["user_id"]: i for i in DetailInfoService.get_list_detail(None, user_id_list=[i["user_id"] for i in enroll_list])}
        for j in enroll_list:
            j["enroll_user_info"] = user_info_map.get(j["user_id"], {})
        # 拼接发起人用户信息
        user_info_map = {i["user_id"]: i for i in DetailInfoService.get_list_detail(None, user_id_list=[i["enroll_user_id"] for i in enroll_list])}
        for j in enroll_list:
            j["initiator_info"] = user_info_map.get(j["user_id"], {})
        # 拼接信息表
        thread_map, err = ThreadListService.search([i["thread_id"] for i in enroll_list], True)
        for j in enroll_list:
            j["enroll_info"] = thread_map.get(j["thread_id"], {})
        # 返回数据
        return {'total': count, "page": page, "size": size, 'list': enroll_list}, None

    @staticmethod
    def record_edit(params: dict = None, pk: int = None):
        params, is_pass = force_transform_type(variable=params, var_type="dict")
        enroll_id, is_pass = force_transform_type(variable=params.pop("enroll_id", None), var_type="int")
        pk, is_pass = force_transform_type(variable=params.pop("id", pk), var_type="int")
        if not pk:
            return None, "msg:非法请求，没有可修改的报名记录;tip:此参数错误，pk不能为空"

        # 流程安全数据验证
        record_obj = EnrollRecord.objects.filter(id=pk).values("enroll_id", "id", "user_id")
        record_info = record_obj.first()
        if not record_info:
            return None, "没有找到id为" + str(pk) + "的记录"
        if enroll_id and not enroll_id == record_info.get("enroll_id"):
            return None, "报名ID不可修改，报名ID为(" + str(enroll_id) + ")的报名中不存在ID为(" + str(pk) + ")的记录"

        # 扩展字段映射获取
        filed_map_list = list(EnrollRecordExtendField.objects.all().values("field", 'field_index'))
        reversal_filed_map = {
            item['field']: item['field_index'] for item in filed_map_list if item['field_index'] in EnrollRecordServices.enroll_record_extend_fields
        }

        # 修改内容检查处理
        try:
            params = format_params_handle(
                param_dict=params,
                is_validate_type=True,
                alias_dict=reversal_filed_map,
                is_remove_empty=True,
                filter_filed_list=list(reversal_filed_map.keys()) + [
                    "apply_subitem_ids|list_int",
                    "enroll_auth_status|int",
                    "enroll_pay_status|int",
                    "enroll_status_code|int",
                    "create_time|date",
                    "price|float",
                    "deposit|float",
                    "count|int",
                    "main_amount|float",
                    "coupon_amount|float",
                    "again_reduction|float",
                    "subitems_amount|float",
                    "deposit_amount|float",
                    "amount|float",
                    "paid_amount|float",
                    "unpaid_amount|float",
                    "fee|float",
                    "photos|json",
                    "files|json",
                    "score|float",
                    "reply",
                    "remark",
                    "finish_time|date",
                    "again_price|float",
                    "again_fee|float",
                    "initiator_again_price|float",
                    "estimated_time|date",
                    "update_time|date",
                ]
            )
        except ValueError as e:
            return None, str(e)

        # 数据进行需改
        try:
            EnrollRecord.objects.filter(id=pk).update(**params)
        except Exception as e:
            return None, "修改异常:" + str(e)
        return None, None

    @staticmethod
    def record_del(pk, search_params=None):
        if pk:
            record_obj = EnrollRecord.objects.filter(id=pk)
        elif search_params and isinstance(search_params, dict):
            record_obj = EnrollRecord.objects.filter(**search_params)
        else:
            return None, "找不到要删除的数据"

        if not record_obj:
            return None, None
        try:
            record_obj.delete()
        except Exception as e:
            return None, "删除异常:" + str(e)
        return None, None

    @staticmethod
    def record_detail(pk: int = None, search_params: dict = None, filter_fields: "str|list" = None):
        """
        报名记录详情
        :param filter_fields: 过滤字段
        :param pk: 查询主键
        :param search_params: 添加搜索，但是仅仅返回第一条
        :return: date, err
        """
        search_params, is_pass = force_transform_type(variable=search_params, var_type="dict", default={})
        # 扩展字段映射获取
        filed_map_list = list(EnrollRecordExtendField.objects.all().values("field", 'field_index'))
        filed_map = {item['field_index']: item['field'] for item in filed_map_list}
        # =================== 构建ORM搜索数据 ==============================
        main_record_obj = EnrollRecord.objects.annotate(
            thread_id=F("enroll__thread_id"),
            enroll_remark=F("enroll__remark")
        )
        if search_params:
            main_record_detail = main_record_obj.filter(**search_params).first()
        else:
            main_record_detail = main_record_obj.filter(id=pk).first()
        # 报名记录不存在，返回给用户提示
        if not main_record_detail:
            return None, "不存在该报名记录"
        main_record_detail = EnrollRecordListSerializer(main_record_detail).data
        # =================== 构建ORM搜索数据 ==============================

        # =================== 记录表字段处理 ==============================
        # 扩展字段还原
        main_record_detail = format_params_handle(
            param_dict=main_record_detail,
            is_remove_null=False,
            alias_dict=filed_map
        )
        # 移除不需要的扩展字段,以及冲突字段别名处理
        main_record_detail = format_params_handle(
            param_dict=main_record_detail,
            is_remove_null=False,
            remove_filed_list=EnrollRecordServices.enroll_record_extend_fields,
            alias_dict={"enroll": "enroll_id"}
        )
        # =================== 记录表处理 ==============================

        # =================== section 多表信息并拼接 start ==============================
        # 分项记录拼接
        main_record_detail["subitem_record_list"], err = EnrollSubitemRecordService.list({"enroll_record_id": pk}, False)
        # 信息表
        thread_id = main_record_detail.get("thread_id", None)
        if thread_id:
            thread_info, thread_err = ThreadItemService.detail(pk=thread_id)
            thread_info = conflict_fields(source_data=main_record_detail, foreign_data=thread_info, prefix="thread_")
            if isinstance(thread_info, dict):
                main_record_detail.update(thread_info)

        # 获取用户表
        user_id = main_record_detail["user_id"]
        if user_id:
            user_info_list = DetailInfoService.get_list_detail(
                user_id_list=[main_record_detail["user_id"]],
                filter_fields=['real_name', 'avatar', 'user_name', 'full_name', 'nickname', 'phone']
            )
            # 拼接用户信息
            if not err and user_info_list and isinstance(user_info_list, list):
                user_info = user_info_list[0]
                user_info = conflict_fields(source_data=main_record_detail, foreign_data=user_info, prefix="record_")
                main_record_detail.update(user_info)
        # =================== section 多表信息并拼接 end   ==============================

        # ====================== 扩展字段过滤  start ==================================
        # 扩展字段过滤 不涉及ORM。所以字段不强制验证，仅仅使用 default_field_list限制即可。
        filter_filed_list = filter_fields_handler(
            input_field_expression=filter_fields,
            default_field_list=list(main_record_detail.keys())
        )
        # 用户u自定义过滤
        main_record_detail = format_params_handle(
            param_dict=main_record_detail,
            is_remove_null=False,
            filter_filed_list=filter_filed_list
        )
        # ====================== 扩展字段过滤  end  ==================================
        return main_record_detail, None

    @staticmethod
    def appoint(enroll_id: int = None, record_id: int = None, subitem_id: int = None, **kwargs):
        """
        指派报名记录于报名分项关系，生成一条报名分项记录。
        :param enroll_id: 报名ID
        :param record_id: 记录ID
        :param subitem_id: 分项ID
        :return: None，err_msg
        """
        # 参数校验
        if enroll_id is None or record_id is None or subitem_id is None:
            return None, "参数错误"

        # 状态码常量
        draft_code, is_pass = force_transform_type(variable=kwargs.get("draft_code", 124), var_type="int", default=124)  # 草稿code
        enrolled_code, is_pass = force_transform_type(variable=kwargs.get("enrolled_code", 356), var_type="int", default=356)  # 已报名code
        enrolling_code, is_pass = force_transform_type(variable=kwargs.get("enrolling_code", 234), var_type="int", default=234)  # 报名中code

        try:
            # 获取报名主体，确认传的enroll_id是否存在
            enroll_query_set = Enroll.objects.filter(id=enroll_id, enroll_status_code__regex=enrolling_code).first()
            if not enroll_query_set:
                return None, "没有找到可以报名的报名主体"

            # 报名记录检查是否存在
            this_record_obj = EnrollRecord.objects.filter(id=record_id).exclude(enroll_status_code=draft_code).first()
            if not this_record_obj:
                return None, "找不到该报名人记录"
            this_record_json = this_record_obj.to_json()

            # 检查是否和报名者的预期相符合
            apply_subitem_ids = this_record_json.get("apply_subitem_ids", None)
            if apply_subitem_ids and isinstance(apply_subitem_ids, list) and not subitem_id in apply_subitem_ids:
                return None, "和报名者的预期并不相符，无法指派该该分类，请支配报名者预期的分项"

            # 校验报名分项是否存在
            enroll_subitem_obj = EnrollSubitem.objects.filter(id=subitem_id).first()
            if not enroll_subitem_obj:
                return None, "subitem_id错误，找不到可以指派的任务"

            # 判断当前的任务是否已经被指派过
            enroll_subitem_record_obj = EnrollSubitemRecord.objects.filter(enroll_subitem_id=subitem_id) \
                .exclude(enroll_subitem_status_code=draft_code).first()
            if not enroll_subitem_record_obj is None:
                return None, "该任务已经指派过了，无法在指派给其他人。"

            # 判断是否剩余可指派的任务
            need_count = enroll_query_set.count or 0  # 需求份数
            enrolled_count = EnrollSubitemRecord.objects.annotate(enroll_id=F("enroll_record__enroll_id")) \
                .filter(enroll_id=enroll_id).exclude(enroll_subitem_status_code=draft_code).values("id").count()
            if enrolled_count >= int(need_count):  # 报名成功的数量
                return None, "已经没有剩余的任务了，无法再次指派"

            # 指派某个人做哪一个任，生成任务记录（生成一条报名分项记录）
            insert_params = {
                "enroll_record_id": record_id,
                "enroll_subitem_id": subitem_id,
                "user_id": this_record_obj.user_id,
                "enroll_subitem_status_code": enrolled_code,
                "price": enroll_subitem_obj.price,
                "count": enroll_subitem_obj.count,
                "subitem_amount": enroll_subitem_obj.price
            }
            EnrollSubitemRecord.objects.create(**insert_params)

            # 判断是否多个任务指派给一个人，修改报名记录得到count, count表示这个报名人拥有几个任务（分项记录） 修改当前的报名记录未被指派状态，变成已经指派
            subitem_record_count = EnrollSubitemRecord.objects.filter(enroll_record_id=record_id).count()
            subitems_amount = (this_record_obj.price or 0 + this_record_obj.again_price or 0) * subitem_record_count
            # 更新当前的报名记录和指派的分项
            EnrollRecord.objects.filter(id=record_id).update(enroll_status_code=enrolled_code, count=subitem_record_count, amount=subitems_amount)
            EnrollSubitem.objects.filter(enroll_id=enroll_id).update(enroll_subitem_status_code=enrolled_code)

            # 再次查询 报名成功的数量, 判断是否可以进入补差补差价。
            # 查询当前指派的人数，和需求的数量，如果相同则指派满，<报名中> ==>> <已报名>
            again_enrolled_count = EnrollSubitemRecord.objects.annotate(enroll_id=F("enroll_record__enroll_id")).filter(
                enroll_id=enroll_id,
                enroll_subitem_status_code=enrolled_code
            ).values("id").count()
            if again_enrolled_count >= need_count:
                # 把剩余没有指派的人全部变成草稿状态
                EnrollRecord.objects.filter(enroll_id=enroll_id).exclude(enroll_status_code=enrolled_code).update(enroll_status_code=draft_code)
                # 如果是一人被指派多份的时候就会出现问题，所以应该使用左连接，以报名分项记录为主表连接报名记录表，enroll_id为搜索
                valuate_result, err = ValuationService.valuation_detailed_list(enroll_id)
                write_to_log(level="info", prefix="指派计价结果：", content="enroll_id:" + str(enroll_id) + "valuate_result:" + str(valuate_result or ""))
                if err:
                    write_to_log(level="error", prefix="报名指派重新计价异常：", content=err)
                    return None, err
                # 报名主表更新
                Enroll.objects.filter(id=enroll_id).update(
                    enroll_status_code=enrolled_code,
                    unpaid_amount=valuate_result["unpaid_amount"],
                    amount=valuate_result["amount"],
                    commision=valuate_result["commision"],
                    subitems_amount=valuate_result["subitems_amount"],
                )
            return None, None

        except Exception as e:
            write_to_log(level="error", prefix="报名指派异常", err_obj=e)
            return None, str(e)

    @staticmethod
    def re_appoint(
            enroll_id: int = None,
            subitem_id: int = None,
            old_record_id: int = None,
            new_record_id: int = None,
            **kwargs
    ):
        """
        重新指派，已报名报名记录和报名分项记录变成草稿，
        :param subitem_id: 原来的分项ID
        :param old_record_id: 原指派的报名记录
        :param new_record_id: 重新指派的报名记录
        :param enroll_id: 报名ID
        :return: None，err_msg
        """
        # 参数校验
        enroll_id, is_pass = force_transform_type(variable=enroll_id, var_type="int")
        old_record_id, is_pass = force_transform_type(variable=old_record_id, var_type="int")
        subitem_id, is_pass = force_transform_type(variable=subitem_id, var_type="int")
        new_record_id, is_pass = force_transform_type(variable=new_record_id, var_type="int")
        if enroll_id is None or old_record_id is None or new_record_id is None or subitem_id is None:
            return None, "参数错误"
        # 状态码常量
        draft_code, is_pass = force_transform_type(variable=kwargs.get("draft_code", 124), var_type="int", default=124)  # 草稿code
        enrolled_code, is_pass = force_transform_type(variable=kwargs.get("enrolled_code", 356), var_type="int", default=356)  # 已报名code
        enrolling_code, is_pass = force_transform_type(variable=kwargs.get("enrolling_code", 234), var_type="int", default=234)  # 报名中code
        try:
            # ================= section 边界验证 start =============================
            # 获取报名主体，确认传的enroll_id是否存在
            enroll_query_set = Enroll.objects.filter(id=enroll_id, enroll_status_code__in=[enrolling_code, enrolled_code]).first()
            if not enroll_query_set:
                return None, "没有找到可以报名的报名主体"

            # 判断当前状态是否在报名中或者已报名
            enroll_dict = enroll_query_set.to_json()
            current_enroll_status_code = enroll_dict.get("enroll_status_code", None)
            if not current_enroll_status_code in [enrolling_code, enrolled_code]:
                return None, "当前的状态不可以重新指派"

            # 报名记录检查是否存在
            old_record = EnrollRecord.objects.filter(id=old_record_id).exclude(enroll_status_code=draft_code).first()
            new_record = EnrollRecord.objects.filter(id=old_record_id).first()
            if not old_record or not new_record:
                return None, "不是有效的报名记录ID"

            # 校验报名分项是否存在
            old_enroll_subitem_obj = EnrollSubitem.objects.filter(id=subitem_id, enroll_id=enroll_id).first()
            if not old_enroll_subitem_obj:
                return None, "subitem_id错误，找不到可以指派的任务"
            # ================= section 边界验证 start =============================

            # ================= section 边界验证通过，进行重新指派 start =============================
            # note 已报名未选中的状态码为草稿，报名中未选中的报名记录状态码是报名中（多份会走该逻辑）
            update_code = draft_code if current_enroll_status_code == enrolled_code else current_enroll_status_code
            # 修改报名记录
            EnrollRecord.objects.filter(id=old_record_id).update(enroll_status_code=update_code)
            EnrollRecord.objects.filter(id=new_record_id).update(enroll_status_code=enrolled_code)
            # 报名分项记录修改
            EnrollSubitemRecord.objects.filter(
                enroll_subitem_id=subitem_id,
                enroll_record_id=old_record_id
            ).update(enroll_record_id=new_record_id)
            # ================= section 边界验证通过，进行重新指派 end   =============================
            return None, None

        except Exception as e:
            write_to_log(level="error", prefix="报名指派异常", err_obj=e)
            return None, str(e)

    @staticmethod
    def old_appoint(enroll_id, record_id, subitem_id):
        """
        镖行天下，报名人指派。(在接入流程控制的老版本指派接口) TODO 后面将一处old接口
        :param enroll_id: 报名ID
        :param record_id: 记录ID
        :param subitem_id: 分项ID
        :return: None，err_msg
        """
        # 定义常量
        appointed_code = 356  # 报名记录被指派的状态码，PS：当前认为356是报名的状态码，报名完成状态
        appointed_start = "^3"
        make_up_difference_code = 243  # 补差价状态码
        make_up_difference_start = "^2"
        rough_draft_code = 124  # 草稿状态码
        rough_draft_start = "^1"
        try:
            # 获取报名主体，确认传的enroll_id是否存在
            enroll_query_set = Enroll.objects.filter(id=enroll_id, enroll_status_code__regex=make_up_difference_start).first()
            if not enroll_query_set:
                return None, "没有找到报名主体"

            # 报名记录检查是否存在
            this_record_obj = EnrollRecord.objects.filter(id=record_id).first()
            if not this_record_obj:
                return None, "找不到该报名人记录"

            # 判断这个报名用户是否被重复指派，暂不限制
            # if this_record_obj.enroll_status_code == appointed_code:
            #     return None, "当前报名人已经指派过了，请勿重复指派"

            # 校验报名分项是否存在
            enroll_subitem_obj = EnrollSubitem.objects.filter(id=subitem_id).first()
            if not enroll_subitem_obj:
                return None, "subitem_id错误，找不到可以指派的任务"

            # 判断是否剩余可指派的任务
            need_count = enroll_query_set.count or 0  # 需求份数
            enrolled_count = EnrollSubitemRecord.objects.annotate(enroll_id=F("enroll_record__enroll_id")) \
                .filter(
                enroll_id=enroll_id,
                enroll_subitem_status_code__regex=appointed_start
            ).values("id").count()  # 报名成功的数量
            if enrolled_count >= int(need_count):
                return None, "已经没有剩余的任务了，无法再次指派"

            # 判断当前的任务是否已经被指派过
            enroll_subitem_record_obj = EnrollSubitemRecord.objects.filter(enroll_subitem_id=subitem_id).exclude(enroll_subitem_status_code__regex=rough_draft_start).first()
            if not enroll_subitem_record_obj is None:
                return None, "该任务已经指派过了，无法在指派给其他人。"

            # 修改当前的报名记录为被指派状态
            EnrollRecord.objects.filter(id=record_id).update(enroll_status_code=appointed_code)

            # 指派某个人做哪一个任，生成任务记录（生成一条报名分项记录）
            insert_params = {
                "enroll_record_id": record_id,
                "enroll_subitem_id": subitem_id,
                "user_id": this_record_obj.user_id,
                "enroll_subitem_status_code": appointed_code,
                "price": enroll_subitem_obj.price,
                "count": enroll_subitem_obj.count,
                "subitem_amount": enroll_subitem_obj.price
            }
            EnrollSubitemRecord.objects.create(**insert_params)

            # 判断是否多个任务指派给一个人，修改报名记录得到count, count表示这个报名人拥有几个任务（分项记录）
            subitem_record_count = EnrollSubitemRecord.objects.filter(enroll_record_id=record_id).count()
            subitems_amount = (this_record_obj.price or 0 + this_record_obj.initiator_again_price or 0) * subitem_record_count
            EnrollRecord.objects.filter(id=record_id).update(count=subitem_record_count, amount=subitems_amount)

            # 再次查询 报名成功的数量, 判断是否可以进入补差补差价。
            again_enrolled_count = EnrollSubitemRecord.objects.annotate(enroll_id=F("enroll_record__enroll_id")) \
                .filter(
                enroll_id=enroll_id,
                enroll_subitem_status_code__regex=appointed_start
            ).values("id").count()

            # 查询当前指派的人数，和需求的数量，如果相同则指派满，则进入下一阶段（代补差价）
            if again_enrolled_count >= need_count:
                # 把剩余没有指派的人全部变成草稿状态
                EnrollRecord.objects.filter(enroll_id=enroll_id).exclude(enroll_status_code__regex=appointed_code).update(enroll_status_code=rough_draft_code)
                # 如果是一人被指派多份的时候就会出现问题，所以应该使用左连接，以报名分项记录为主表连接报名记录表，enroll_id为搜索
                valuate_result, err = ValuationService.valuation_detailed_list(enroll_id)
                write_to_log(level="info", prefix="指派计价结果：", content="enroll_id:" + str(enroll_id) + "valuate_result:" + str(valuate_result or ""))
                if err:
                    write_to_log(level="error", prefix="报名指派重新计价异常：", content=err)

                # 报名主表更新
                Enroll.objects.filter(id=enroll_id).update(
                    unpaid_amount=valuate_result["unpaid_amount"],
                    amount=valuate_result["amount"],
                    commision=valuate_result["commision"],
                    subitems_amount=valuate_result["subitems_amount"],
                )
                # 修改全部的
                EnrollStatusCodeService.batch_edit_code(enroll_id=enroll_id, code=make_up_difference_code)
            return None, None

        except Exception as e:
            write_to_log(level="error", prefix="报名指派异常", err_obj=e)
            return None, str(e)



# encoding: utf-8
"""
@project: djangoModel->valuation_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 计价生成服务
@created_time: 2022/10/13 9:40
"""
import sys

from django.db.models import F

from ..models import Enroll, EnrollRuleValuate
# 接口服务类
from ..service.subitem_service import SubitemService
from ..utils.j_valuation import JExpression


class ValuationService:
    expression = None  # 表达式

    @staticmethod
    def valuate_test(expression_string=None, input_dict=None):
        # 测试代码
        if input_dict is None:
            input_dict = {}

        expression_string, parsed_variable_map = JExpression.parse_variables(
            expression_string,
            input_dict
        )
        calculator = JExpression()
        data, err = calculator.process(expression_string)
        return data, err

    @staticmethod
    def valuate(category_id=None, classify_id=None, enroll_rule_group_id=None, variables_dict=None):
        # 获取表达式
        if category_id and classify_id:
            expression_list = EnrollRuleValuate.objects \
                .annotate(category_id=F("enroll_rule_group__category_id")) \
                .annotate(classify_id=F("enroll_rule_group__classify_id")) \
                .filter(category_id=category_id, classify_id=classify_id) \
                .order_by("-sort") \
                .values("field", "expression_string")
        else:
            expression_list = EnrollRuleValuate.objects \
                .annotate(category_id=F("enroll_rule_group__category_id")) \
                .annotate(classify_id=F("enroll_rule_group__classify_id")) \
                .filter(enroll_rule_group_id=enroll_rule_group_id) \
                .order_by("-sort") \
                .values("field", "expression_string")

        expression_map = {i["field"]: i["expression_string"] for i in expression_list}
        result = {}
        for k, v in expression_map.items():
            expression_string, parse_err = JExpression.parse_variables(
                v,
                variables_dict
            )

            calculator = JExpression()
            data, err = calculator.process(expression_string)
            data = round(data, 2) if isinstance(data, float) else data
            result.update({k: data})
            variables_dict[k] = data
        return result, None

    @staticmethod
    def valuation_detailed_list(enroll_id):
        # 动态加载模块
        if not getattr(sys.modules.get("xj_enroll.service.enroll_record_serivce"), "EnrollRecordServices"):
            from xj_enroll.service.enroll_record_serivce import EnrollRecordServices
        else:
            EnrollRecordServices = getattr(sys.modules.get("xj_enroll.service.enroll_record_serivce"), "EnrollRecordServices")

        enroll_obj = Enroll.objects.filter(id=enroll_id)
        if not enroll_obj:
            return None, "不存在该报名信息"
        # 变量字典
        variables_dict = enroll_obj.first().to_json()

        # 获取计价ID
        enroll_rule_group_id = variables_dict.get("enroll_rule_group_id")
        if not enroll_rule_group_id:
            return None, "该报名没有绑定计价ID"

        # 获取计价规则
        valuate_obj = EnrollRuleValuate.objects.filter(enroll_rule_group_id=enroll_rule_group_id).order_by("-sort")
        if not valuate_obj:
            return None, "没有配置计价公式"

        # 分项的变量
        subitem_list, err = SubitemService.list({"enroll_id": enroll_id}, False)
        subitem_dict = {}
        for i in subitem_list:
            for k, v in i.items():
                key = "subitem__" + k
                value = str(v) if v else "0"
                subitem_dict[key] = value if (subitem_dict.get(key, None) is None) else (subitem_dict[key] + "," + value)
        variables_dict.update(subitem_dict)

        # 记录的变量
        record_list, err = EnrollRecordServices.record_list({"enroll_id": enroll_id}, False)
        record_dict = {}
        for i in record_list:
            for j, f in i.items():
                key = "record__" + j
                value = str(f) if f else "0"
                record_dict[key] = value if (record_dict.get(key, None) is None) else (record_dict[key] + "," + value)
        variables_dict.update(record_dict)

        # 计算公式解析
        result = {}
        valuate_list = valuate_obj.values("name", "type", "field", "expression_string")
        for item in valuate_list:
            expression_string, parsed_variable_map = JExpression.parse_variables(
                item["expression_string"],
                variables_dict
            )
            calculator = JExpression()
            data, err = calculator.process(expression_string)
            variables_dict[item["field"]] = data
            result[item["field"]] = data
        return result, None



# encoding: utf-8
"""
@project: djangoModel->enroll_statistics
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 统计接口
@created_time: 2022/10/31 11:07
"""
from django.views.decorators.http import require_http_methods
from rest_framework.views import APIView

from xj_finance.services.finance_transact_service import FinanceTransactService
from xj_user.utils.user_wrapper import user_authentication_force_wrapper
from ..service.enroll_statistics_services import EnrollStatisticsServices
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper


class EnrollStatisticsAPI(APIView):
    @require_http_methods(['GET'])
    @request_params_wrapper
    def every_one_total(self, *args, request_params=None, **kwargs):
        data, err = EnrollStatisticsServices.every_one_total(params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @require_http_methods(['GET'])
    @request_params_wrapper
    def every_day_total(self, *args, request_params=None, **kwargs):
        data, err = EnrollStatisticsServices.every_day_total(params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @require_http_methods(['GET'])
    @request_params_wrapper
    def statistics(self, *args, request_params=None, **kwargs):
        data, err = EnrollStatisticsServices.statistics_by_day()
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @require_http_methods(['GET'])
    @user_authentication_force_wrapper
    def statistics_by_user(self, *args, user_info=None, **kwargs):
        if user_info is None:
            user_info = {}
        user_id = user_info.get("user_id")
        data, err = EnrollStatisticsServices.statistics_by_user(user_id)
        finance_res, err = FinanceTransactService.get_finance_by_user(user_id)
        if err:
            return util_response(err=1000, msg=err)
        data["balance"] = round(float(finance_res.get("balance", 0)), 2) if finance_res else 0
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)



# encoding: utf-8
"""
@project: djangoModel->valuation_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 报名计时服务
@created_time: 2022/10/13 9:40
"""
from django.core.paginator import Paginator, EmptyPage
from django.db import transaction
from django.db.models import F

from xj_user.services.user_detail_info_service import DetailInfoService
from ..models import EnrollSubitemRecord, EnrollRecord, EnrollSubitem
from ..utils.custom_tool import format_params_handle, force_transform_type


class EnrollSubitemRecordService:

    @staticmethod
    def add(params: dict = None):
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        exclude_code, is_pass = force_transform_type(variable=params.get("exclude_code"), var_type="int", default=124)
        # ================ section 参数验证 start =====================
        try:
            # 参数类型校验
            params = format_params_handle(
                param_dict=params,
                is_validate_type=True,
                is_remove_empty=True,
                filter_filed_list=[
                    "enroll_record_id|int", "enroll_subitem_id|int", "user_id|int", "price|float", "count|int", "subitem_amount|float",
                    "enroll_subitem_status_code|int", "reply", "remark", "files|dict", "photos|dict"
                ],
            )
            # 参数必填验证
            must_key = ["enroll_record_id", "enroll_subitem_id", "user_id"]
            for k in must_key:
                if not params.get(k):
                    return None, "参数错误：" + k + "必填"
            # 报名记录与报名分项确认是否存在
            if not EnrollRecord.objects.filter(id=params.get("enroll_record_id")).first():
                return None, "不存在记录ID为" + str(params.get("enroll_record_id")) + "报名记录"
            subitem_obj = EnrollSubitem.objects.filter(id=params.get("enroll_subitem_id")).first()
            if not subitem_obj:
                return None, "不存在分项ID为" + str(params.get("enroll_subitem_id")) + "报名分项"
            # 防止重复添加
            if EnrollSubitemRecord.objects.filter(
                    enroll_record_id=params.get("enroll_record_id"),
                    enroll_subitem_id=params.get("enroll_subitem_id"),
                    user_id=params.get("user_id")
            ).exclude(enroll_subitem_status_code=exclude_code).first():
                return None, "已经生成分项记录，无需再次生成"
        except ValueError as e:
            return None, str(e)
        # ================ section 参数验证 end   =====================
        # IO操作
        try:
            instance = EnrollSubitemRecord.objects.create(**params)
            add_info = instance.to_json()
            return {"id": add_info.get("id"), "enroll_id": subitem_obj.enroll_id}, None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def edit(params=None, pk=None):
        # ================ section 数据合法型验证 start =====================
        pk, is_pass = force_transform_type(variable=params.pop("id", pk), var_type="int")
        if not pk:
            return None, "不是有效的修改主键"

        # 检查是否存在该分项记录数据
        query_obj = EnrollSubitemRecord.objects.annotate(
            enroll_id=F("enroll_subitem__enroll_id")
        ).filter(id=pk).values("id", "enroll_id")
        if not query_obj:
            return None, "没有可修改的数据"

        # 流程合法型校验
        query_obj_first = query_obj.first()
        enroll_id, is_pass = force_transform_type(variable=params.pop("enroll_id", None), var_type="int")
        if enroll_id and not enroll_id == query_obj_first.get("enroll_id"):
            return None, "该报名(" + str(enroll_id) + ")不存在ID为(" + str(pk) + ")的分项记录"

        # 字段合法性校验, 禁止修改分项、记录、用户的ID
        try:
            params = format_params_handle(
                param_dict=params,
                is_remove_empty=True,
                is_validate_type=True,
                filter_filed_list=[
                    "price|float", "count|int", "subitem_amount|float",
                    "enroll_subitem_status_code|int", "reply", "remark", "files|dict", "photos|dict"
                ]
            )
        except ValueError as e:
            return None, str(e)
        # ================ section 数据合法型验证 end   =====================
        # IO操作
        try:
            EnrollSubitemRecord.objects.filter(id=pk).update(**params)
        except Exception as e:
            return None, "修改异常:" + str(e)
        return None, None

    @staticmethod
    def list(params, need_pagination=True):
        # 字段处理
        size = params.pop('size', 10)
        page = params.pop('page', 1)
        params['enroll_status_code'] = params['enroll_status_code'].split(";") if params.get("enroll_status_code") else None
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "enroll_record_id", "enroll_record_id__in", "enroll_subitem_id", "enroll_subitem_id__in", "user_id", "price", "count", "subitem_amount",
                "enroll_subitem_status_code", "reply", "remark", "files", "photos", "enroll_id"
            ],
            alias_dict={"enroll_status_code": "enroll_status_code__in"}
        )
        # 全部展示
        query_obj = EnrollSubitemRecord.objects.annotate(enroll_id=F("enroll_record__enroll_id")).filter(**params).values()
        if not need_pagination:
            res_list = list(query_obj)
            # 拼接用户信息
            user_ids = [i["user_id"] for i in res_list]
            user_info_map = {i["user_id"]: i for i in DetailInfoService.get_list_detail(None, user_id_list=user_ids)}
            for j in res_list:
                j["user_info"] = user_info_map.get(j["user_id"], {})

            return res_list, None

        paginator = Paginator(query_obj, size)
        # 分页展示
        try:
            paginator_obj = paginator.page(page)
        except EmptyPage:
            return {'total': paginator.count, "page": page, "size": size, 'list': []}, None
        except Exception as e:
            print("e", e)
            return None, f'{str(e)}'
        # 拼接用户信息
        res_list = list(paginator_obj.object_list)
        user_ids = [i["user_id"] for i in res_list]
        user_info_map = {i["user_id"]: i for i in DetailInfoService.get_list_detail(None, user_id_list=user_ids)}
        for j in res_list:
            j["user_info"] = user_info_map.get(j["user_id"], {})

        return {'total': paginator.count, "page": page, "size": size, 'list': res_list}, None

    @staticmethod
    def delete(pk, search_params=None):
        """取消报名"""
        if not search_params:
            subrecord_obj = EnrollSubitemRecord.objects.filter(id=pk)
        else:
            try:
                subrecord_obj = EnrollSubitemRecord.objects.filter(**search_params)
            except Exception as e:
                return None, "搜索参数不正确"

        if not subrecord_obj:
            return None, "没有可修改得到数据"

        try:
            subrecord_obj.delete()
        except Exception as e:
            return None, "删除异常:" + str(e)
        return None, None

    @staticmethod
    def batch_copy(pk=None, copy_params=None, copy_num=None, ):
        """
        批量复制报名分项记录
        :param copy_params: 复制插入的参数
        :param pk: 搜索ID主键
        :param copy_count: 复制参数
        :return: data,err
        """
        if not copy_num:
            return None, "拷贝的分数不能为空，至少为1分"

        if not pk and not copy_params:
            return None, "找不到可以复制的记录"

        # 开始事务
        sid = transaction.savepoint()
        try:
            if not copy_params or isinstance(copy_params, dict):
                copy_params_obj = EnrollSubitemRecord.objects.filter(id=pk)
                if not copy_params_obj:
                    return None, "找不到可以复制的记录"
                copy_params = copy_params_obj.first().to_json()

            # 遍历插入数据
            for i in range(copy_num):
                EnrollSubitemRecord.objects.create(**copy_params)

            # 清除保存点
            transaction.clean_savepoints()
            return None, None

        except Exception as e:
            print("err:", str(e))
            transaction.savepoint_rollback(sid)
            return None, str(e)



# encoding: utf-8
"""
@project: djangoModel->category_api
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 小分类API
@created_time: 2022/9/20 11:21
"""
from django.views.decorators.http import require_http_methods
from rest_framework.views import APIView

from ..service.classify_service import ClassifyService
from ..utils.model_handle import parse_data, util_response
from ..validator.category_validator import CategoryValidator


class ClassifyApi(APIView):
    @require_http_methods(['GET'])
    def list(self, *args, **kwargs, ):
        params = parse_data(self)
        data, err = ClassifyService.list(params=params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @require_http_methods(['POST'])
    def add(self, *args, **kwargs, ):
        params = parse_data(self)
        # 表单数据验证
        is_valid, error = CategoryValidator(params).validate()
        if not is_valid:
            return util_response(err=1000, msg=error)
        # 添加数据
        data, err = ClassifyService.add(params)
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)

    @require_http_methods(['PUT'])
    def edit(self, *args, **kwargs, ):
        params = parse_data(self)
        classify_id = kwargs.get("classify_id") or params.pop("classify_id") or None
        if not classify_id:
            return util_response(err=1000, msg="参数错误:enroll_id不可以为空")
        data, err = ClassifyService.edit(params, classify_id)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @require_http_methods(['DELETE'])
    def delete(self, *args, **kwargs, ):
        params = parse_data(self)
        classify_id = kwargs.get("classify_id") or params.pop("classify_id") or None
        if not classify_id:
            return util_response(err=1000, msg="参数错误:enroll_id不可以为空")
        data, err = ClassifyService.edit(params, classify_id)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)



from rest_framework.decorators import api_view
from rest_framework.views import APIView

from xj_user.services.user_detail_info_service import DetailInfoService
from ..models import Enroll
from ..service.enroll_record_serivce import EnrollRecordServices
from ..service.valuation_service import ValuationService
from ..utils.custom_response import util_response
from ..utils.custom_tool import parse_data, flow_service_wrapper, write_to_log, force_transform_type
from ..utils.custom_tool import request_params_wrapper
from ..utils.join_list import JoinList
from ..utils.user_wrapper import user_authentication_force_wrapper, user_authentication_wrapper


class RecordAPI(APIView):
    # 添加记录,用户报名
    @api_view(['POST'])
    @user_authentication_force_wrapper
    @request_params_wrapper
    @flow_service_wrapper
    def add(self, *args, user_info=None, request_params=None, **kwargs, ):
        # 提取参数
        request_params, is_pass = force_transform_type(variable=request_params, var_type="dict", default={})
        request_params.setdefault("user_id", user_info.get("user_id", None))  # note 用户ID判断，不允许匿名报名，但是可以代报名（指派）。
        if not request_params.get("user_id"):
            return util_response(err=1006, msg="没有找到有效的用户ID")
        enroll_id, is_pass = force_transform_type(variable=request_params.get("enroll_id"), var_type="int")

        # ===============  section 检查报名ID是否正确  start ================
        if not enroll_id:
            return util_response(err=1000, msg="没有找到有效的报名ID")
        enroll_info = Enroll.objects.filter(id=enroll_id).first()
        if not enroll_info:
            return util_response(err=1001, msg="不是一个有效的报名ID")
        # ===============  section 检查报名ID是否正确  end   ================

        # ===============  section 入库之前进行计价  start ================
        try:
            request_params.setdefault("again_price", 0)
            valuation_res, err = ValuationService.valuate(
                enroll_rule_group_id=enroll_info.enroll_rule_group_id or 1,
                variables_dict=request_params
            )
            request_params["initiator_again_price"] = valuation_res.get("initiator_again_price", 0)
            write_to_log(prefix="报名记录添加，计价结果", content="enroll_info:" + str(enroll_info) + "valuation_res:" + str(valuation_res))
        except Exception as e:
            write_to_log(prefix="报名记录添加触发计价错误", err_obj=e)
        # ===============  section 入库之前进行计价  end   ================

        # ===============  section 添加数据  start ================
        data, err = EnrollRecordServices.record_add(request_params)
        if err:
            return util_response(err=1002, msg=err)
        return util_response(data=data)
        # ===============  section 添加数据  end   ================

    @api_view(['GET'])
    @user_authentication_wrapper
    @request_params_wrapper
    def list(self, *args, request_params, user_info, **kwargs, ):
        # 参数转杯
        need_pagination, is_pass = force_transform_type(variable=request_params.get("need_pagination", 1), var_type="bool", default=True)
        exclude_code, is_pass = force_transform_type(variable=request_params.get("exclude_code"), var_type="int", default=124)
        look_self, is_pass = force_transform_type(variable=request_params.get("look_self"), var_type="bool", default=False)
        if look_self and user_info:
            request_params["user_id"] = user_info.get("user_id")
        elif look_self and not user_info:
            return util_response(err=6001, msg="非法请求，请您登录")
        # 查询列表
        data, err = EnrollRecordServices.record_list(
            params=request_params,
            need_pagination=need_pagination,
            exclude_code=exclude_code
        )
        if err:
            return util_response(err=1000, msg=err)
        user_ids = []
        if need_pagination:
            user_infos = DetailInfoService.get_list_detail({}, user_ids)
            data["list"] = JoinList(data["list"], user_infos, "user_id", "user_id").join()
        else:
            user_infos = DetailInfoService.get_list_detail({}, user_ids)
            data = JoinList(data, user_infos, "user_id", "user_id").join()

        return util_response(data=data)

    @api_view(['GET'])
    def list_v2(self, *args, **kwargs, ):
        params = parse_data(self)
        data, err = EnrollRecordServices.complex_record_list(params=params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(['DELETE'])
    @user_authentication_force_wrapper
    def record_del(self, *args, **kwargs, ):
        params = parse_data(self) or {}
        pk = kwargs.get("pk") or params.pop("id")
        data, err = EnrollRecordServices.record_del(pk)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(['PUT'])
    @user_authentication_force_wrapper
    @request_params_wrapper
    @flow_service_wrapper
    def record_edit(self, *args, request_params, **kwargs, ):
        pk = kwargs.get("pk", None) or request_params.pop("id", None) or request_params.pop("record_id", None)
        pk, is_pass = force_transform_type(variable=pk, var_type="int")
        if not pk:
            return util_response(err=1000, msg="不是有效的报名记录主键")

        data, err = EnrollRecordServices.record_edit(request_params, pk)
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)

    @api_view(['GET'])
    @user_authentication_force_wrapper
    def record_detail(self, *args, **kwargs, ):
        params = parse_data(self) or {}
        pk = kwargs.get("pk", None) or params.pop("id", None) or params.pop("record_id", None) or None
        if not pk:
            return util_response(err=1000, msg="参数错误")
        data, err = EnrollRecordServices.record_detail(pk, search_params=params, filter_fields=params.pop("filter_fields", None))

        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)

    @api_view(['GET'])
    @user_authentication_force_wrapper
    @request_params_wrapper
    @flow_service_wrapper
    def appoint(self, *args, user_info=None, request_params, **kwargs, ):
        """
        需求描述：报名可多人报名，不在自动成单，采用手动指派报名
        1.由用户或者客服进行指派工作人员完成任务。
        2.没有被选中的用户，报名状态修改成草稿状态。
        3.主报名项目，则进入代补差价状态。
        :return: response
        """
        # ========== section 参数校验 start==========
        enroll_id = request_params.pop("enroll_id", None)
        record_id = request_params.pop("record_id", None)
        subitem_id = request_params.pop("subitem_id", None)
        new_record_id = request_params.pop("new_record_id", None)
        if enroll_id is None or record_id is None or subitem_id is None:
            return util_response(err=1000, msg="参数错误")
        # ========== section 参数校验 end  ==========
        if not new_record_id:
            data, err = EnrollRecordServices.appoint(enroll_id, record_id, subitem_id, **request_params)
        else:
            if record_id == new_record_id:
                return util_response(err=1001, msg="无法重复指派给同一个人")
            data, err = EnrollRecordServices.re_appoint(
                enroll_id=enroll_id,
                subitem_id=subitem_id,
                old_record_id=record_id,
                new_record_id=new_record_id,
                **request_params
            )
        if err:
            return util_response(err=1002, msg=err)
        return util_response(data=data, msg="指派成功")

    @api_view(['GET'])
    @user_authentication_force_wrapper
    @request_params_wrapper
    @flow_service_wrapper
    def old_appoint(self, *args, user_info=None, **kwargs, ):
        """
        需求描述：报名可多人报名，不在自动成单，可以手动指派报名
        1.由用户或者客服进行指派工作人员完成任务。
        2.没有被选中的用户，报名状态修改成草稿状态。
        3.主报名项目，则进入代补差价状态。
        :return: response
        """
        params = parse_data(self)
        enroll_id = params.get("enroll_id", None)
        record_id = params.get("record_id", None)
        subitem_id = params.get("subitem_id", None)
        if enroll_id is None or record_id is None or subitem_id is None:
            return util_response(err=1000, msg="参数错误")
        data, err = EnrollRecordServices.old_appoint(enroll_id, record_id, subitem_id)
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data, msg="指派成功")



# encoding: utf-8
"""
@project: djangoModel->enroll_status_code_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 报名状态码服务
@created_time: 2022/11/13 16:41
"""
from django.db import transaction

from ..models import Enroll, EnrollRecord, EnrollSubitem, EnrollSubitemRecord
from ..utils.custom_tool import filter_fields_handler, force_transform_type


class EnrollStatusCodeService():
    @staticmethod
    def batch_edit_code(enroll_id: int = None, code: int = None, edit_types: "str|list" = None, exclude_code: int = 124, params: dict = None, **kwargs):
        """
        流程控制，批量修改状态码服务方法
        :param exclude_code: 不查询的该状态code的数据
        :param enroll_id: 报名ID
        :param code: 修改的状态码
        :param edit_types: 需要修改策略列表
        :param params: 需要差异化修改的参数
        :return: data, err
        """
        # ==================== section 参数处理 ================================
        enroll_id, is_pass = force_transform_type(variable=enroll_id, var_type="int")
        if not enroll_id:
            return None, "不是有效的报名ID"

        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        kwargs, is_pass = force_transform_type(variable=kwargs, var_type="dict", default={})
        params.update(kwargs)

        edit_types = filter_fields_handler(
            input_field_expression=edit_types,
            all_field_list=["enroll", "enroll_record", "enroll_subitem", "enroll_subitem_record"],
            split_char=kwargs.get("split_char", ";")
        )
        code, is_pass = force_transform_type(variable=code, var_type="int")
        exclude_code, is_pass = force_transform_type(variable=exclude_code, var_type="int", default=124)
        # ==================== section 参数处理 ================================

        # ==================== section 开启事务进行修改 start ==================
        sid = transaction.savepoint()
        try:
            # 修改报名主表
            if "enroll" in edit_types:
                enroll_query = Enroll.objects.filter(id=enroll_id)
                if not enroll_query.first():
                    return None, "这是一个不存在的报名ID"
                enroll_query.update(enroll_status_code=code)

            # 修改报名记录
            if "enroll_record" in edit_types:
                enroll_record_query_obj = EnrollRecord.objects
                if exclude_code:
                    enroll_record_query_obj = enroll_record_query_obj.exclude(enroll_status_code=exclude_code)
                if params.get("enroll_record_id"):
                    enroll_record_query_obj = enroll_record_query_obj.filter(id=params["enroll_record_id"])
                else:
                    enroll_record_query_obj = enroll_record_query_obj.filter(enroll_id=enroll_id)
                enroll_record_query_obj.update(enroll_status_code=params.get("enroll_record_code", code))

            # 修改报名分项
            if "enroll_subitem" in edit_types:
                enroll_subitem_query_obj = EnrollSubitem.objects
                if exclude_code:
                    enroll_subitem_query_obj = enroll_subitem_query_obj.exclude(enroll_subitem_status_code=exclude_code)
                if params.get("enroll_subitem_id"):
                    enroll_subitem_query_obj = enroll_subitem_query_obj.filter(id=params["enroll_subitem_id"])
                else:
                    enroll_subitem_query_obj = enroll_subitem_query_obj.filter(enroll_id=enroll_id)
                enroll_subitem_query_obj.update(enroll_subitem_status_code=params.get("enroll_subitem_code", code))

            # 修改报名分项记录
            if "enroll_subitem_record" in edit_types:
                enroll_record_query_obj = EnrollSubitemRecord.objects
                if exclude_code:
                    enroll_record_query_obj = enroll_record_query_obj.exclude(enroll_subitem_status_code=exclude_code)
                if params.get("enroll_subitem_record_id"):
                    enroll_subitem_record_query_obj = enroll_record_query_obj.filter(id=params["enroll_subitem_record_id"])
                else:
                    enroll_subitem_record_query_obj = EnrollSubitemRecord.objects.filter(enroll_record__enroll_id=enroll_id)
                enroll_subitem_record_query_obj.update(enroll_subitem_status_code=params.get("enroll_subitem_record_code", code))

            transaction.clean_savepoints()
            return None, None
            # ==================== section 开启事务进行修改 end ===================
        except Exception as e:
            transaction.savepoint_rollback(sid)
            return None, str(e)



# encoding: utf-8
"""
@project: djangoModel->enroll_status_code
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 所有状态相关操作接口
@created_time: 2022/11/13 16:31
"""
from django.views.decorators.http import require_http_methods
from rest_framework.views import APIView

from xj_user.utils.user_wrapper import user_authentication_force_wrapper
from ..service.enroll_status_code_service import EnrollStatusCodeService
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper, flow_service_wrapper, force_transform_type, dynamic_load_class, write_to_log


class EnrollStatusCodeAPI(APIView):
    @require_http_methods(['PUT'])
    @user_authentication_force_wrapper
    @request_params_wrapper
    @flow_service_wrapper
    def batch_edit_code(self, *args, request_params=None, user_info=None, **kwargs, ):
        code = request_params.pop("code", None)
        if not code:
            return util_response(err=1000, msg="参数错误，code|enroll_id 不能为空")
        enroll_id = request_params.pop("enroll_id", None) or kwargs.get("enroll_id", None)
        edit_types = request_params.pop("edit_types", None)
        data, err = EnrollStatusCodeService.batch_edit_code(
            enroll_id=enroll_id,
            code=code,
            edit_types=edit_types,
            params=request_params
        )
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)

    @require_http_methods(['GET'])
    @user_authentication_force_wrapper
    @request_params_wrapper
    @flow_service_wrapper
    def ask_pay_status(self, *args, request_params=None, user_info=None, **kwargs, ):
        # 参数处理
        # print("request_params:", request_params)
        success_code = request_params.get("success_code", )
        fail_code = request_params.get("fail_code")
        edit_types = request_params.get("edit_types", "enroll;enroll_record;enroll_subitem;enroll_subitem_record")
        enroll_id, is_pass = force_transform_type(variable=request_params.get("enroll_id"), var_type="int")
        if not enroll_id:
            return util_response(err=1000, msg="msg:参数错误，报名ID不能为空:tip:非法请求")
        if not success_code or not fail_code:
            return util_response(err=1001, msg="msg:请检亲求参数，或者检查流程配置:tip:非法请求")

        # note 询问资金模块是完成了订单 得到 WATING(等待中),SUCCESS（支付成功）,FAIL（支付失败）三种状态
        PaymentService, import_err = dynamic_load_class(import_path="xj_payment.services.payment_service", class_name="PaymentService")
        if import_err:
            return util_response(err=1002, msg="该系统并没有安装支付模块")

        try:
            order_info, err = PaymentService.ask_order_status(params={
                "enroll_id": enroll_id
            })
            if err:
                pay_status = "FAIL"
            else:
                pay_status = order_info.get("status", "FAIL")

        except Exception as e:
            write_to_log(prefix="订单支付的状态查询异常", err_obj=e)
            return util_response(err=1004, msg="订单支付的状态查询异常请查看详细日志")

        if pay_status == "WATING":
            return util_response(msg="msg:正在支付中;tip:正在支付中", data=pay_status)
        elif pay_status == "SUCCESS":
            code = success_code
        elif pay_status == "FAIL":
            code = fail_code
        else:
            return util_response(msg="msg:支付模块异常，并没有给到支付状态;tip:正在支付中", data=pay_status)

        # 修改报名的全部状态
        data, err = EnrollStatusCodeService.batch_edit_code(
            enroll_id=enroll_id,
            code=code,
            edit_types=edit_types.split("edit_types"),
            params=request_params
        )
        if err:
            return util_response(err=1005, msg=err)
        return util_response(data=data)



# encoding: utf-8
"""
@project: djangoModel->subitem_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 报名分项记录
@created_time: 2022/10/15 12:38
"""

from django.core.paginator import Paginator, EmptyPage
from django.db.models import F

from ..models import EnrollSubitem, EnrollSubitemExtendField, Enroll, EnrollSubitemRecord
from ..serializers import EnrollSubitemSerializer
from ..service.enroll_subitem_record_service import EnrollSubitemRecordService
from ..service.subitem_extend_service import input_convert, output_convert
from ..utils.custom_tool import format_params_handle, force_transform_type


class SubitemService:
    @staticmethod
    def add(params: dict = None, **kwargs):
        """
        分项添加
        :param params: 添加参数
        :return: None，err
        """
        params, is_pass = force_transform_type(variable=params, var_type="only_dict", default={})
        kwargs, is_pass = force_transform_type(variable=kwargs, var_type="only_dict", default={})
        params.update(kwargs)
        # ============= section 报名ID合法性判断 start ==================
        exclude_code, is_pass = force_transform_type(variable=params.get("exclude_code"), var_type="int")
        enroll_id, is_pass = force_transform_type(variable=params.get("enroll_id"), var_type="int")
        if not enroll_id:
            return None, "请填写报名ID"
        enroll_obj = Enroll.objects.filter(id=enroll_id)
        if exclude_code:
            enroll_obj = enroll_obj.exclude(enroll_status_code=exclude_code)
        enroll_info = enroll_obj.first()
        if not enroll_info:
            return None, "报名不存在"
        # ============= section 报名ID合法性判断 end   ==================

        # ============= section 参数处理 start ==================
        params = input_convert(
            params_dict=params,
            enroll_id=enroll_id
        )
        try:
            params = format_params_handle(
                param_dict=params,
                is_validate_type=True,
                is_remove_empty=True,
                filter_filed_list=[
                    "enroll_id|int",
                    "name",
                    "price|float",
                    "count|int",
                    "unit",
                    "amount|float",
                    "enroll_subitem_status_code|int",
                    "description",
                    "remark",
                    "field_1",
                    "field_2",
                    "field_3",
                    "field_4",
                    "field_5",
                    "field_6",
                    "field_7",
                    "field_8",
                    "field_9",
                    "field_10",
                ]
            )
        except ValueError as e:
            return None, str(e)
        # ============= section 参数处理 end ==================
        try:
            instance = EnrollSubitem.objects.create(**params)
            add_info = instance.to_json()
            return {"enroll_id": enroll_id, "id": add_info.get("id")}, None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def edit(params: dict = None, subitem_id=None):
        # 参数验证
        params, is_pass = force_transform_type(variable=params, var_type="only_dict", default={})
        subitem_id, is_pass = force_transform_type(variable=params.pop("id", subitem_id), var_type="int", default=0)
        subitem_obj_first = EnrollSubitem.objects.filter(id=subitem_id).first()
        if not subitem_obj_first:
            return None, "找不到ID为" + str(subitem_id) + "的数据"
        subitem_info = subitem_obj_first.to_json()

        # 流程验证，同时剔除报名ID。报名分项不允许更改报名ID。
        enroll_id, is_pass = force_transform_type(variable=params.pop("enroll_id", None), var_type="int")  # 流程控制得到报名ID
        if enroll_id and not enroll_id == subitem_info.get("enroll_id"):
            return None, "报名ID不可修改，报名ID为(" + str(enroll_id) + ")的报名不存在ID为(" + str(subitem_id) + ")的分项"

        # ============= section 参数处理 start ==================
        params = input_convert(
            params_dict=params,
            enroll_id=subitem_info.get("enroll_id")
        )
        try:
            params = format_params_handle(
                param_dict=params,
                is_validate_type=True,
                is_remove_empty=True,
                filter_filed_list=[
                    "name",
                    "price|float",
                    "count|int",
                    "unit",
                    "amount|float",
                    "enroll_subitem_status_code|int",
                    "description",
                    "remark",
                    "field_1",
                    "field_2",
                    "field_3",
                    "field_4",
                    "field_5",
                    "field_6",
                    "field_7",
                    "field_8",
                    "field_9",
                    "field_10",
                ]
            )
        except ValueError as e:
            return None, str(e)
        # ============= section 参数处理 end ==================

        # 开始修改
        try:
            EnrollSubitem.objects.filter(id=subitem_id).update(**params)
        except Exception as e:
            return None, "修改异常:" + str(e)

        # 联动修改全部报名相关得到状态，迁移到报名服务中，通过流程控制
        # unfinish_count = EnrollSubitem.objects.filter(enroll_id=enroll_id).exclude(enroll_subitem_status_code=668).exclude(enroll_subitem_status_code=124).count()
        # if unfinish_count == 0 and enroll_id:
        #     EnrollStatusCodeService.batch_edit_code(enroll_id, params.get("enroll_subitem_status_code", 668))
        #     try:
        #         # ============ 完成订单联动资金修改 start ============
        #         # TODO 资金联动代码块，后期使用流程控制
        #         commision = Enroll.objects.filter(id=enroll_id).values("commision").first()
        #         commision = commision.get("commision") if commision else 0
        #         records_vales = list(EnrollRecord.objects.filter(enroll_id=enroll_id,enroll_status_code=668).values("user_id", "price", "initiator_again_price"))
        #         for item in records_vales:
        #             write_to_log(prefix="资金联动修改：", content={
        #                 "account_id": item.get("user_id"),
        #                 "amount": commision or 0,
        #                 "currency": "CNY",
        #                 "pay_mode": "BALANCE",
        #                 "enroll_id": enroll_id
        #             })
        #             FinanceTransactsService.finance_create_or_write_off(data={
        #                 "account_id": item.get("user_id"),
        #                 "amount": commision or 0,
        #                 "currency": "CNY",
        #                 "pay_mode": "BALANCE",
        #                 "enroll_id": enroll_id
        #             })
        #         # ============ 完成订单联动资金修改 end ============
        #     except Exception as e:
        #         write_to_log(prefix="资金联动修改：", err_obj=e)
        return None, None

    @staticmethod
    def detail(pk=None):
        # 参数验证
        subitem_obj = EnrollSubitem.objects.annotate(category_id=F("enroll__category_id")).filter(id=pk).first()
        if not subitem_obj:
            return None, "找不到ID为" + str(pk) + "的数据"

        subitem_dict = EnrollSubitemSerializer(subitem_obj, many=False).data
        subitem_list = output_convert([subitem_dict])
        subitem_detail = subitem_list[0] if subitem_list else {}

        subitem_detail["subitem_record_list"] = list(EnrollSubitemRecord.objects.filter(enroll_subitem_id=pk).values())
        return subitem_detail, None

    @staticmethod
    def list(params: dict = None, is_pagination=True):
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        is_pagination, is_pass = force_transform_type(variable=is_pagination, var_type="bool", default=True)
        size, is_pass = force_transform_type(variable=params.pop('size', 10), var_type="int", default=10)
        page, is_pass = force_transform_type(variable=params.pop('page', 1), var_type="int", default=1)
        # 字段过滤
        params = format_params_handle(
            param_dict=params,
            is_remove_empty=True,
            filter_filed_list=[
                "id|int", "category_id|int", "enroll_subitem_status_code|int", "enroll_subitem_status_codes|list_int",
                "enroll_id|int", "name", "price|float", "count|int", "unit", "description", "remark"
            ],
            split_list=["enroll_subitem_status_codes"],
            alias_dict={"name": "name__contains", "enroll_subitem_status_codes": "enroll_subitem_status_code__in"}
        )
        try:
            # 不分页查询
            fetch_obj = EnrollSubitem.objects.annotate(category_id=F("enroll__category_id")).filter(**params).values()
            total = fetch_obj.count()
            if not is_pagination and total <= 2000:
                result_list = output_convert(list(fetch_obj))
                # 拼接分项记录
                subitem_ids = [i["id"] for i in result_list]
                res_list, err = EnrollSubitemRecordService.list({"enroll_subitem_id__in": subitem_ids}, False)
                subitem_record_map = {}
                for item in res_list:
                    if subitem_record_map.get(item["enroll_subitem_id"]):
                        subitem_record_map[item["enroll_subitem_id"]].append(item)
                    else:
                        subitem_record_map[item["enroll_subitem_id"]] = [item]
                for item in result_list:
                    item["subitem_record_list"] = subitem_record_map.get(item.get("id"))
                return result_list, None
            else:
                # 分页查询
                paginator = Paginator(fetch_obj, size)
                try:
                    page_obj = paginator.page(page)
                except EmptyPage:
                    return {'total': total, "page": page, "size": size, 'list': []}, None
                result_list = list(page_obj.object_list)
                result_list = output_convert(result_list)
                # 拼接分项记录
                subitem_ids = [i["id"] for i in result_list]
                res_list, err = EnrollSubitemRecordService.list({"enroll_subitem_id__in": subitem_ids}, False)
                subitem_record_map = {}
                for item in res_list:
                    if subitem_record_map.get(item["enroll_subitem_id"]):
                        subitem_record_map[item["enroll_subitem_id"]].append(item)
                    else:
                        subitem_record_map[item["enroll_subitem_id"]] = [item]
                return {'total': total, "size": size, 'page': page, 'list': result_list}, None
        except Exception as e:
            return [], str(e)

    # 批量修改
    @staticmethod
    def batch_edit(params, enroll_id=None):
        enroll_id = enroll_id or params.pop("enroll_id", None)
        if not enroll_id:
            return None, "请填写报名ID"

        # 参数根据类别转化
        params = input_convert(
            params_dict=params,
            enroll_id=enroll_id
        )
        if not params:
            return None, "enroll_id不能为空，或者参数为空"

        subitem_enroll_obj = EnrollSubitem.objects.filter(enroll_id=enroll_id)
        if not subitem_enroll_obj:
            return None, "没有找到enroll_id为" + str(enroll_id) + "的报名分项"
        try:
            subitem_enroll_obj.update(**params)
        except Exception as e:
            return None, "修改参数错误:" + str(e)
        return None, None

    @staticmethod
    def delete(subitem_rule_id):
        subitem_enroll_obj = EnrollSubitem.objects.filter(id=subitem_rule_id)
        if not subitem_enroll_obj:
            return None, None
        try:
            subitem_enroll_obj.delete()
        except Exception as e:
            return None, "删除异常:" + str(e)
        return None, None

    @staticmethod
    def extend_field(params=None, is_pagination=True):
        validate_params = params if isinstance(params, dict) else {}

        size = validate_params.pop('size', 10)
        page = validate_params.pop('page', 1)

        filtered_params = format_params_handle(
            param_dict=validate_params,
            filter_filed_list=["id", "category_id", "field_index", "field", "label", "type", "config", "description", ],
            alias_dict={"field": "field__contains", "label": "label__contains"}
        )

        try:
            extend_obj = EnrollSubitemExtendField.objects.all()
            extend_obj_list = extend_obj.filter(**filtered_params).values()
            if not is_pagination:
                return list(extend_obj_list), None

            paginator = Paginator(extend_obj_list, size)
            paginator_obj_list = paginator.page(page)
            data = {'total': paginator.count, "size": size, 'page': page, 'list': list(paginator_obj_list.object_list)}
            return data, None
        except Exception as e:
            return [], "查询参数错误：" + str(e)

    @staticmethod
    def check_num(enroll_id):
        enroll_obj = Enroll.objects.filter(id=enroll_id).first()
        if not enroll_obj:
            return False
        enroll_count = enroll_obj.to_json().get("count")
        subitem_count = EnrollSubitem.objects.filter(enroll_id=enroll_id).count()
        if enroll_count > subitem_count:
            return True
        return False
